import EventEmitter from "eventemitter3"
import type { GameEvent } from "../shared/gameEvents"
import type { Interaction, InteractionQueueItem } from "../shared/script/types"
import { script } from "../shared/script"
import { getInteractionQueueItemId } from "../shared/util"

interface Events {
    "game-event": [GameEvent]
}

export class Game extends EventEmitter<Events> {
    private currentRoomId: string = script.roomsById.values().next()!.value!.id
    private interactionQueue: Map<string, InteractionQueueItem> = new Map()

    constructor() {
        super()
    }

    getInitialStateEvents(): GameEvent[] {
        const events: GameEvent[] = []

        events.push({ type: "room-changed", roomId: this.currentRoomId })
        this.interactionQueue.forEach(item => events.push({ type: "interaction-queued", item }))

        return events
    }

    addInteractionVote(interaction: Interaction) {
        const id = getInteractionQueueItemId(interaction)
        const existingItem = this.interactionQueue.get(id)
        if (existingItem === undefined) {
            const item = {
                id,
                votes: 1,
                interaction
            }

            this.interactionQueue.set(item.id, item)
            this.emit("game-event", { type: "interaction-queued", item })
        } else {
            existingItem.votes += 1
            this.emit("game-event", { type: "interaction-votes-changed", id: existingItem.id, votes: existingItem.votes })
        }
    }

    removeInteractionVote(id: string) {
        const item = this.interactionQueue.get(id)
        if (item !== undefined) {
            item.votes -= 1
            this.emit("game-event", { type: "interaction-votes-changed", id: item.id, votes: item.votes })

            if (item.votes <= 0) this.interactionQueue.delete(id)
        }
    }

    switchRoom(roomId: string) {
        this.currentRoomId = roomId
        this.interactionQueue.clear()
        this.emit("game-event", { type: "room-changed", roomId })
    }

    removeInteractionQueueItem(id: string) {
        if (!this.interactionQueue.has(id)) return
        this.emit("game-event", { type: "interaction-votes-changed", id: id, votes: 0 })
        this.interactionQueue.delete(id)
    }
}

export const game = new Game()