import createExpressApp from "express"
import { listen } from "listhen"
import { WebSocketServer } from "ws"
import { appRouter } from "./trpc"
import { createExpressMiddleware as createTrpcMiddleware } from "@trpc/server/adapters/express"
import { applyWSSHandler } from "@trpc/server/adapters/ws"
import { createContext } from "./trpc/base"
import { isDev } from "./isDev"

const expressApp = createExpressApp()

expressApp.use("/trpc", createTrpcMiddleware({
    router: appRouter,
    createContext: ({ req, res }) => createContext(res),
}))

const { server } = await listen(expressApp, { isProd: !isDev, autoClose: false })

const wss = new WebSocketServer({ server, path: "/ws" })
const wssTrpcHandler = applyWSSHandler({
    wss,
    router: appRouter,
    createContext: () => createContext(undefined)
})

const stop = () => {
    console.log("Received stop signal")
    server.close()
    wssTrpcHandler.broadcastReconnectNotification()
    wss.close(console.error)
}

process.on("SIGTERM", stop)
process.on("SIGINT", stop)

process.on("exit", () => {
    console.log("exit")
})