import { z } from "zod"

export interface Script {
    roomsById: Map<string, Room>
}

export interface Room {
    id: string
    label: string
    initialObjects: Set<GameObject>
    hiddenObjects: Set<GameObject>
    combinations: Set<Combination>
}

export interface GameObject {
    id: string
    label: string
}

export interface Combination {
    id: string
    inputs: Set<{
        objectId: string
        isConsumed: boolean
    }>
    outputIds: Set<string>
}

export const interactionSchema = z.discriminatedUnion("type", [
    z.object({
        type: z.literal("use"),
        objectId: z.string()
    }),
    z.object({
        type: z.literal("combine"),
        objectIds: z.set(z.string())
    })
])

export type Interaction = z.infer<typeof interactionSchema>

export interface InteractionQueueItem {
    id: string
    votes: number
    interaction: Interaction
}
